@@ -5,7 +5,7 @@ from numbers import Number
 import numpy as np
 from packaging.version import Version
 
-from unyt import delta_degC
+from unyt import delta_degC, dimensionless
 from unyt.array import NULL_UNIT, unyt_array, unyt_quantity
 from unyt.dimensions import temperature
 from unyt.exceptions import (
@@ -81,14 +81,14 @@ def dot(a, b, out=None):
 
 
 @implements(np.vdot)
-def vdot(a, b):
+def vdot(a, b, /):
     return np.vdot._implementation(np.asarray(a), np.asarray(b)) * (
         getattr(a, "units", NULL_UNIT) * getattr(b, "units", NULL_UNIT)
     )
 
 
 @implements(np.inner)
-def inner(a, b):
+def inner(a, b, /):
     return np.inner._implementation(np.asarray(a), np.asarray(b)) * (
         getattr(a, "units", NULL_UNIT) * getattr(b, "units", NULL_UNIT)
     )
@@ -158,20 +158,42 @@ def _sanitize_range(_range, units):
     return new_range.squeeze()
 
 
+def _sanitize_bins(arr, bins):
+    if np.isscalar(bins):
+        # integer giving number of bins
+        if hasattr(bins, "units"):
+            if bins.to_value(dimensionless).is_integer():
+                return int(bins.to_value(dimensionless))
+            else:
+                raise ValueError("Bin count must be a dimensionless integer.")
+        else:
+            return bins
+    if hasattr(bins, "units"):
+        # if bins has units, must be convertible to arr's units (assumed dimensionless if
+        # absent)
+        return bins.to_value(getattr(arr, "units", dimensionless))
+    elif hasattr(arr, "units"):
+        # bins doesn't have units but arr does, only ok if arr is dimensionless
+        return bins / arr.units.get_conversion_factor(dimensionless)[0]
+    # neither arr nor bins have units (but maybe weights or density do)
+    return bins
+
+
 def _histogram(a, *, bins=10, range=None, density=None, weights=None, normed=None):
     range = _sanitize_range(range, units=[getattr(a, "units", None)])
+    sanitized_bins = _sanitize_bins(a, bins)
     if NUMPY_VERSION >= Version("1.24"):
-        counts, bins = np.histogram._implementation(
+        counts, sanitized_bins = np.histogram._implementation(
             np.asarray(a),
-            bins=bins,
+            bins=sanitized_bins,
             range=range,
             density=density,
             weights=np.asarray(weights) if weights is not None else None,
         )
     else:
-        counts, bins = np.histogram._implementation(
+        counts, sanitized_bins = np.histogram._implementation(
             np.asarray(a),
-            bins=bins,
+            bins=sanitized_bins,
             range=range,
             normed=normed,
             weights=np.asarray(weights) if weights is not None else None,
@@ -184,7 +206,9 @@ def _histogram(a, *, bins=10, range=None, density=None, weights=None, normed=Non
         counts /= a.units
     if weights is not None and hasattr(weights, "units"):
         counts *= weights.units
-    return counts, bins * getattr(a, "units", 1)
+    return counts, (
+        sanitized_bins * getattr(a, "units", 1) if np.isscalar(bins) else bins
+    )
 
 
 if NUMPY_VERSION >= Version("1.24"):
@@ -206,20 +230,28 @@ def _histogram2d(x, y, *, bins=10, range=None, density=None, weights=None, norme
     range = _sanitize_range(
         range, units=[getattr(x, "units", None), getattr(y, "units", None)]
     )
+    if np.isscalar(bins):
+        sanitized_xbins, sanitized_ybins = (
+            _sanitize_bins(x, bins),
+            _sanitize_bins(y, bins),
+        )
+    else:
+        sanitized_xbins = _sanitize_bins(x, bins[0])
+        sanitized_ybins = _sanitize_bins(y, bins[1])
     if NUMPY_VERSION >= Version("1.24"):
-        counts, xbins, ybins = np.histogram2d._implementation(
+        counts, obtained_xbins, obtained_ybins = np.histogram2d._implementation(
             np.asarray(x),
             np.asarray(y),
-            bins=bins,
+            bins=(sanitized_xbins, sanitized_ybins),
             range=range,
             density=density,
             weights=np.asarray(weights) if weights is not None else None,
         )
     else:
-        counts, xbins, ybins = np.histogram2d._implementation(
+        counts, obtained_xbins, obtained_ybins = np.histogram2d._implementation(
             np.asarray(x),
             np.asarray(y),
-            bins=bins,
+            bins=(sanitized_xbins, sanitized_ybins),
             range=range,
             normed=normed,
             weights=np.asarray(weights) if weights is not None else None,
@@ -235,7 +267,19 @@ def _histogram2d(x, y, *, bins=10, range=None, density=None, weights=None, norme
             counts /= y.units
     if weights is not None and hasattr(weights, "units"):
         counts *= weights.units
-    return counts, xbins * getattr(x, "units", 1), ybins * getattr(y, "units", 1)
+    return (
+        counts,
+        (
+            obtained_xbins * getattr(x, "units", 1)
+            if np.isscalar(sanitized_xbins)
+            else bins[0]
+        ),
+        (
+            obtained_ybins * getattr(y, "units", 1)
+            if np.isscalar(sanitized_ybins)
+            else bins[1]
+        ),
+    )
 
 
 if NUMPY_VERSION >= Version("1.24"):
@@ -265,18 +309,24 @@ def _histogramdd(
     sample, *, bins=10, range=None, density=None, weights=None, normed=None
 ):
     range = _sanitize_range(range, units=[getattr(_, "units", None) for _ in sample])
+    if np.isscalar(bins):
+        sanitized_bins = [_sanitize_bins(a, bins) for a in sample]
+    else:
+        sanitized_bins = [
+            _sanitize_bins(a, b) for a, b in zip(sample, bins, strict=True)
+        ]
     if NUMPY_VERSION >= Version("1.24"):
-        counts, bins = np.histogramdd._implementation(
+        counts, obtained_bins = np.histogramdd._implementation(
             [np.asarray(_) for _ in sample],
-            bins=bins,
+            bins=sanitized_bins,
             range=range,
             density=density,
             weights=np.asarray(weights) if weights is not None else None,
         )
     else:
-        counts, bins = np.histogramdd._implementation(
+        counts, obtained_bins = np.histogramdd._implementation(
             [np.asarray(_) for _ in sample],
-            bins=bins,
+            bins=sanitized_bins,
             range=range,
             normed=normed,
             weights=np.asarray(weights) if weights is not None else None,
@@ -296,7 +346,12 @@ def _histogramdd(
 
     if weights is not None and hasattr(weights, "units"):
         counts *= weights.units
-    return counts, tuple(_bin * getattr(s, "units", 1) for _bin, s in zip(bins, sample))
+    return counts, tuple(
+        ob * getattr(s, "units", 1) if np.isscalar(sb) else bins[i]
+        for i, (ob, sb, s) in enumerate(
+            zip(obtained_bins, sanitized_bins, sample, strict=True)
+        )
+    )
 
 
 if NUMPY_VERSION >= Version("1.24"):
@@ -375,8 +430,8 @@ def _validate_units_consistency_v2(ref_units, *args) -> None:
 
 
 @implements(np.concatenate)
-def concatenate(arrs, /, axis=0, out=None, *args, **kwargs):
-    ret_units = _validate_units_consistency(arrs)
+def concatenate(arrays, /, axis=0, out=None, *args, **kwargs):
+    ret_units = _validate_units_consistency(arrays)
 
     if out is not None:
         out_view = np.asarray(out)
@@ -384,7 +439,7 @@ def concatenate(arrs, /, axis=0, out=None, *args, **kwargs):
         out_view = out
 
     res = np.concatenate._implementation(
-        [np.asarray(_) for _ in arrs], axis, out_view, *args, **kwargs
+        [np.asarray(_) for _ in arrays], axis, out_view, *args, **kwargs
     )
 
     if getattr(out, "units", None) is not None:
@@ -484,7 +539,8 @@ def around(a, decimals=0, out=None):
 @implements(np.block)
 def block(arrays):
     ret_units = _validate_units_consistency(arrays)
-    return np.block._implementation(arrays) * ret_units
+    res = np.block._implementation(arrays)
+    return unyt_array(res, ret_units, bypass_validation=True)
 
 
 @implements(np.fft.fft)
@@ -649,7 +705,7 @@ def _linspace(
         return result * start.units
 
 
-if NUMPY_VERSION >= Version("2.0.0.dev0"):
+if NUMPY_VERSION >= Version("2.0.0dev0"):
 
     @implements(np.linspace)
     def linspace(
@@ -983,7 +1039,7 @@ def put_along_axis(arr, indices, values, axis, *args, **kwargs) -> None:
 
 
 @implements(np.putmask)
-def putmask(a, mask, values, *args, **kwargs) -> None:
+def putmask(a, /, mask, values, *args, **kwargs) -> None:
     _validate_units_consistency_v2(a.units, values)
     np.putmask._implementation(np.asarray(a), mask, np.asarray(values), *args, **kwargs)
 
@@ -1065,7 +1121,7 @@ else:
 
 
 @implements(np.where)
-def where(condition, *args, **kwargs):
+def where(condition, /, *args, **kwargs):
     if len(args) == 0:
         return np.where._implementation(np.asarray(condition), **kwargs)
 
@@ -1212,7 +1268,7 @@ def trapezoid(y, x=None, dx=1.0, *args, **kwargs):
     )
 
 
-if hasattr(np, "in1d"):
+if NUMPY_VERSION < Version("2.4.0.dev0"):
 
     @implements(np.in1d)  # noqa: NPY201
     def in1d(ar1, ar2, *args, **kwargs):
@@ -1240,3 +1296,55 @@ def take(a, indices, axis=None, out=None, mode="raise"):
 
     ret_cls = unyt_quantity if res.ndim == 0 else unyt_array
     return ret_cls(res, ret_units, bypass_validation=True)
+
+
+if NUMPY_VERSION < Version("2.4.1"):
+
+    def _average(a, axis=None, weights=None, returned=False, *, keepdims=np._NoValue):
+        if NUMPY_VERSION < Version("1.23"):
+            res = np.average._implementation(
+                np.asarray(a),
+                axis=axis,
+                weights=weights,
+                returned=returned,
+            )
+        else:
+            res = np.average._implementation(
+                np.asarray(a),
+                axis=axis,
+                weights=weights,
+                returned=returned,
+                keepdims=keepdims,
+            )
+        if returned:
+            avg, wsum = res
+        else:
+            avg = res
+        if returned and isinstance(weights, unyt_array):
+            wsum_cls = unyt_quantity if wsum.ndim == 0 else unyt_array
+            wsum_units = weights.units
+            wsum = wsum_cls(wsum, wsum_units, bypass_validation=True)
+        if isinstance(a, unyt_array):
+            avg_cls = unyt_quantity if avg.ndim == 0 else unyt_array
+            avg_units = a.units
+            avg = avg_cls(avg, avg_units, bypass_validation=True)
+        if returned:
+            return avg, wsum
+        return avg
+
+
+if NUMPY_VERSION < Version("1.23.0"):
+
+    @implements(np.average)
+    def average(a, axis=None, weights=None, returned=False):
+        return _average(a, axis=axis, weights=weights, returned=returned)
+
+elif NUMPY_VERSION < Version("2.4.1"):
+    # will not be needed once https://github.com/numpy/numpy/pull/30522 is merged,
+    # probably in numpy 2.4.1, but this will still be needed until earlier versions
+    # are no longer supported
+    @implements(np.average)
+    def average(a, axis=None, weights=None, returned=False, *, keepdims=np._NoValue):
+        return _average(
+            a, axis=axis, weights=weights, returned=returned, keepdims=keepdims
+        )
